from typing import Dict, Union, List, Any, Optional
from formant.sdk.utils.time_utils import current_datapoint_time


def with_timestamp(f):
    """Decorator that injects the timestamp kwarg with self._timestamp"""

    def with_timestamp_inner(*args, **kwargs):
        self = args[0]
        if "timestamp" not in kwargs:
            kwargs["timestamp"] = self._timestamp
        return f(*args, **kwargs)

    return with_timestamp_inner


class IngestionRequest:
    def __init__(self, device_id: str, tags: Dict, timestamp: Optional[int] = None):
        self._device_id = device_id
        self._tags = tags
        self._timestamp = timestamp
        if timestamp is None:
            self._timestamp = current_datapoint_time()
        self._request: Dict[str, List] = {"items": []}

    def __str__(self):
        return str(json.dumps(self._request, indent=4))

    def _get_entry(self, type: str, name: str):
        entry = next(
            (
                item
                for item in self._request["items"]
                if item["name"] == name
                and item["deviceId"] == self._device_id
                and item["type"] == type
            ),
            None,
        )
        if entry is None:
            entry = {
                "deviceId": self._device_id,
                "name": name,
                "type": type,
                "tags": self._tags,
                "points": [],
            }
            self._request["items"].append(entry)
        return entry

    def get_request(self):
        return self._request

    @with_timestamp
    def add_numeric(
        self, name: str, value: Union[int, float], timestamp: Optional[int] = None
    ):
        entry = self._get_entry("numeric", name)
        entry["points"].append([timestamp, value])

    @with_timestamp
    def add_text(self, name: str, value: str, timestamp: Optional[int] = None):
        if value == "" or value == "\n":
            return
        entry = self._get_entry("text", name)
        entry["points"].append([timestamp, value])

    @with_timestamp
    def add_json(self, name: str, value: str, timestamp: Optional[int] = None):
        entry = self._get_entry("json", name)
        entry["points"].append([timestamp, value])

    @with_timestamp
    def add_numeric_set(
        self, name: str, value: List[Dict[str, Any]], timestamp: Optional[int] = None
    ):
        entry = self._get_entry("numeric set", name)
        entry["points"].append([timestamp, value])
        return self

    @with_timestamp
    def add_bitset(
        self, name: str, value: Dict[str, List], timestamp: Optional[int] = None
    ):
        if len(value["keys"]) == 0:
            return
        entry = self._get_entry("bitset", name)
        entry["points"].append([timestamp, value])

    @with_timestamp
    def add_battery(
        self,
        name: str,
        percentage: float,
        voltage: Optional[float] = None,
        current: Optional[float] = None,
        charge: Optional[float] = None,
        timestamp: Optional[int] = None,
    ):
        entry = self._get_entry("battery", name)
        value = {"percentage": percentage}
        if voltage is not None:
            value["voltage"] = voltage
        if current is not None:
            value["current"] = current
        if charge is not None:
            value["charge"] = charge
        entry["points"].append([timestamp, value])

    @with_timestamp
    def add_health(self, name: str, timestamp: Optional[int] = None):
        entry = self._get_entry("health", name)
        entry["points"].append(
            [
                [
                    timestamp,
                    {
                        "status": "operational",
                    },
                ]
            ]
        )

    @with_timestamp
    def add_location(
        self,
        name: str,
        longitude: float,
        latitude: float,
        timestamp: Optional[int] = None,
    ):
        entry = self._get_entry("location", name)
        entry["points"].append(
            [
                [
                    timestamp,
                    {
                        "latitude": latitude,
                        "longitude": longitude,
                    },
                ]
            ]
        )

    @with_timestamp
    def add_image(self, name: str, url: str, timestamp: Optional[int] = None):
        entry = self._get_entry("image", name)
        entry["points"] = [
            [
                self._timestamp,
                {
                    "url": url,
                },
            ]
        ]
        entry["points"].append(
            [
                [
                    timestamp,
                    {
                        "url": url,
                    },
                ]
            ]
        )



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.transform import Transform


T = TypeVar("T", bound="Goal")


@attr.s(auto_attribs=True)
class Goal:
    """
    Attributes:
        world_to_local (Transform):
        pose (Transform):
    """

    world_to_local: "Transform"
    pose: "Transform"
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        world_to_local = self.world_to_local.to_dict()

        pose = self.pose.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "worldToLocal": world_to_local,
                "pose": pose,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.transform import Transform

        d = src_dict.copy()
        world_to_local = Transform.from_dict(d.pop("worldToLocal"))

        pose = Transform.from_dict(d.pop("pose"))

        goal = cls(
            world_to_local=world_to_local,
            pose=pose,
        )

        goal.additional_properties = d
        return goal

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import List
from formant.protos.model.v1 import navigation_pb2
from formant.sdk.agent.v1.localization.utils import validate_type, get_ros_module
from .transform import Transform


class Path:
    @classmethod
    def from_ros(cls, path):
        nav_msgs = get_ros_module("nav_msgs.msg")
        validate_type(path, nav_msgs.Path)
        return cls(poses=[Transform.from_ros_pose_stamped(pose) for pose in path.poses])

    def __init__(
        self,
        poses=[],  # type: List[Transform]
        transform_to_world=Transform(),  # type: Transform
    ):
        self.poses = poses
        self.transform_to_world = transform_to_world

    def to_proto(self):
        path_proto = navigation_pb2.Path(
            world_to_local=self.transform_to_world.to_proto(),
        )
        poses_proto_array = [pose.to_proto() for pose in self.poses]
        del path_proto.poses[:]
        path_proto.poses.extend(poses_proto_array)
        return path_proto



from enum import Enum


class UsageMetricMetricType(str, Enum):
    DATA_IN_BYTES = "data-in-bytes"
    DATA_IN_COUNT = "data-in-count"
    DATA_OUT_BYTES = "data-out-bytes"
    DATA_OUT_COUNT = "data-out-count"
    ANALYTICS_OUT_BYTES = "analytics-out-bytes"
    ANALYTICS_OUT_COUNT = "analytics-out-count"
    DATA_STORED_BYTES = "data-stored-bytes"

    def __str__(self) -> str:
        return str(self.value)



from . import (
    Response,
    BeginUploadRequest,
    BeginUploadResponse,
    CompleteUploadRequest,
    Any,
    file_controller_begin_upload,
    file_controller_complete_upload,
)
from formant.sdk.cloud.v2.src.resources.resources import Resources
import ntpath
import os
import requests


class Files(Resources):
    def upload(self, path: str, timeout: int = 60):
        """Uploads a file"""
        file_name = ntpath.basename(path)
        file_size = os.path.getsize(path)
        if not file_size > 0:
            raise ValueError("File is empty")
        begin_upload_request = BeginUploadRequest(file_name, file_size)
        begin_upload_result = self._begin_upload(begin_upload_request, timeout)
        file_id = begin_upload_result.parsed.file_id
        upload_id = begin_upload_result.parsed.upload_id
        part_urls = begin_upload_result.parsed.part_urls
        part_size = begin_upload_result.parsed.part_size
        etags = self._create_etags(path, part_urls, part_size)
        complete_upload_request = CompleteUploadRequest(file_id, upload_id, etags)
        complete_upload_response = self._complete_upload(
            complete_upload_request, timeout
        )
        return complete_upload_response

    def _begin_upload(self, begin_upload_request: BeginUploadRequest, timeout: int):
        client = self._get_client().with_timeout(timeout)
        response: Response[
            BeginUploadResponse
        ] = file_controller_begin_upload.sync_detailed(
            client=client, json_body=begin_upload_request
        )
        return response

    def _complete_upload(
        self, complete_upload_request: CompleteUploadRequest, timeout: int
    ):
        client = self._get_client().with_timeout(timeout)
        response: Response[Any] = file_controller_complete_upload.sync_detailed(
            client=client, json_body=complete_upload_request
        )
        return response

    def _create_etags(self, path: str, part_urls: list, part_size: int):
        etags = []
        part_index = 0
        file_obj = open(path, "rb")
        for part_url in part_urls:
            file_obj.seek(part_index * part_size)
            part_index = part_index + 1
            data = file_obj.read(part_size)
            response = requests.put(part_url, data=data)
            etags.append(response.headers["etag"])
        file_obj.close()
        return etags

    async def upload_async(self, path: str, timeout: int = 60):
        """Uploads a file"""
        file_name = ntpath.basename(path)
        file_size = os.path.getsize(path)
        if not file_size > 0:
            raise ValueError("File is empty")
        begin_upload_request = BeginUploadRequest(file_name, file_size)
        begin_upload_result = await self._begin_upload_async(
            begin_upload_request, timeout
        )
        file_id = begin_upload_result.parsed.file_id
        upload_id = begin_upload_result.parsed.upload_id
        part_urls = begin_upload_result.parsed.part_urls
        part_size = begin_upload_result.parsed.part_size
        etags = self._create_etags(path, part_urls, part_size)
        complete_upload_request = CompleteUploadRequest(file_id, upload_id, etags)
        complete_upload_response = await self._complete_upload_async(
            complete_upload_request, timeout
        )
        return complete_upload_response

    def _begin_upload_async(
        self, begin_upload_request: BeginUploadRequest, timeout: int
    ):
        client = self._get_client().with_timeout(timeout)
        response: Response[
            BeginUploadResponse
        ] = file_controller_begin_upload.asyncio_detailed(
            client=client, json_body=begin_upload_request
        )
        return response

    def _complete_upload_async(
        self, complete_upload_request: CompleteUploadRequest, timeout: int
    ):
        client = self._get_client().with_timeout(timeout)
        response: Response[Any] = file_controller_complete_upload.asyncio_detailed(
            client=client, json_body=complete_upload_request
        )
        return response



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.google_storage_info_output_format import GoogleStorageInfoOutputFormat
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_storage_export import GoogleStorageExport


T = TypeVar("T", bound="GoogleStorageInfo")


@attr.s(auto_attribs=True)
class GoogleStorageInfo:
    """
    Attributes:
        credentials_json_key (str):
        google_storage_export (Union[Unset, GoogleStorageExport]):
        output_format (Union[Unset, GoogleStorageInfoOutputFormat]):
    """

    credentials_json_key: str
    google_storage_export: Union[Unset, "GoogleStorageExport"] = UNSET
    output_format: Union[Unset, GoogleStorageInfoOutputFormat] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        credentials_json_key = self.credentials_json_key
        google_storage_export: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.google_storage_export, Unset):
            google_storage_export = self.google_storage_export.to_dict()

        output_format: Union[Unset, str] = UNSET
        if not isinstance(self.output_format, Unset):
            output_format = self.output_format.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "credentialsJsonKey": credentials_json_key,
            }
        )
        if google_storage_export is not UNSET:
            field_dict["googleStorageExport"] = google_storage_export
        if output_format is not UNSET:
            field_dict["outputFormat"] = output_format

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_storage_export import GoogleStorageExport

        d = src_dict.copy()
        credentials_json_key = d.pop("credentialsJsonKey")

        _google_storage_export = d.pop("googleStorageExport", UNSET)
        google_storage_export: Union[Unset, GoogleStorageExport]
        if isinstance(_google_storage_export, Unset):
            google_storage_export = UNSET
        else:
            google_storage_export = GoogleStorageExport.from_dict(_google_storage_export)

        _output_format = d.pop("outputFormat", UNSET)
        output_format: Union[Unset, GoogleStorageInfoOutputFormat]
        if isinstance(_output_format, Unset):
            output_format = UNSET
        else:
            output_format = GoogleStorageInfoOutputFormat(_output_format)

        google_storage_info = cls(
            credentials_json_key=credentials_json_key,
            google_storage_export=google_storage_export,
            output_format=output_format,
        )

        google_storage_info.additional_properties = d
        return google_storage_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.partial_view_layout_type import PartialViewLayoutType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.filter_ import Filter
    from ..models.partial_view_tags import PartialViewTags
    from ..models.view_configuration import ViewConfiguration


T = TypeVar("T", bound="PartialView")


@attr.s(auto_attribs=True)
class PartialView:
    """
    Attributes:
        organization_id (Union[Unset, str]):
        name (Union[Unset, str]):
        url (Union[Unset, None, str]):
        show_on_single_device (Union[Unset, bool]):
        show_on_multi_device (Union[Unset, bool]):
        show_on_teleop (Union[Unset, bool]):
        show_on_analytics (Union[Unset, bool]):
        show_timeline (Union[Unset, bool]):
        filter_ (Union[Unset, Filter]):
        layout (Union[Unset, Any]):
        layout_type (Union[Unset, None, PartialViewLayoutType]):
        configuration (Union[Unset, List['ViewConfiguration']]):
        index (Union[Unset, int]):
        id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
        tags (Union[Unset, PartialViewTags]):
    """

    organization_id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    url: Union[Unset, None, str] = UNSET
    show_on_single_device: Union[Unset, bool] = UNSET
    show_on_multi_device: Union[Unset, bool] = UNSET
    show_on_teleop: Union[Unset, bool] = UNSET
    show_on_analytics: Union[Unset, bool] = UNSET
    show_timeline: Union[Unset, bool] = UNSET
    filter_: Union[Unset, "Filter"] = UNSET
    layout: Union[Unset, Any] = UNSET
    layout_type: Union[Unset, None, PartialViewLayoutType] = UNSET
    configuration: Union[Unset, List["ViewConfiguration"]] = UNSET
    index: Union[Unset, int] = UNSET
    id: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    tags: Union[Unset, "PartialViewTags"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        organization_id = self.organization_id
        name = self.name
        url = self.url
        show_on_single_device = self.show_on_single_device
        show_on_multi_device = self.show_on_multi_device
        show_on_teleop = self.show_on_teleop
        show_on_analytics = self.show_on_analytics
        show_timeline = self.show_timeline
        filter_: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.filter_, Unset):
            filter_ = self.filter_.to_dict()

        layout = self.layout
        layout_type: Union[Unset, None, str] = UNSET
        if not isinstance(self.layout_type, Unset):
            layout_type = self.layout_type.value if self.layout_type else None

        configuration: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.configuration, Unset):
            configuration = []
            for configuration_item_data in self.configuration:
                configuration_item = configuration_item_data.to_dict()

                configuration.append(configuration_item)

        index = self.index
        id = self.id
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        tags: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id
        if name is not UNSET:
            field_dict["name"] = name
        if url is not UNSET:
            field_dict["url"] = url
        if show_on_single_device is not UNSET:
            field_dict["showOnSingleDevice"] = show_on_single_device
        if show_on_multi_device is not UNSET:
            field_dict["showOnMultiDevice"] = show_on_multi_device
        if show_on_teleop is not UNSET:
            field_dict["showOnTeleop"] = show_on_teleop
        if show_on_analytics is not UNSET:
            field_dict["showOnAnalytics"] = show_on_analytics
        if show_timeline is not UNSET:
            field_dict["showTimeline"] = show_timeline
        if filter_ is not UNSET:
            field_dict["filter"] = filter_
        if layout is not UNSET:
            field_dict["layout"] = layout
        if layout_type is not UNSET:
            field_dict["layoutType"] = layout_type
        if configuration is not UNSET:
            field_dict["configuration"] = configuration
        if index is not UNSET:
            field_dict["index"] = index
        if id is not UNSET:
            field_dict["id"] = id
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at
        if tags is not UNSET:
            field_dict["tags"] = tags

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.filter_ import Filter
        from ..models.partial_view_tags import PartialViewTags
        from ..models.view_configuration import ViewConfiguration

        d = src_dict.copy()
        organization_id = d.pop("organizationId", UNSET)

        name = d.pop("name", UNSET)

        url = d.pop("url", UNSET)

        show_on_single_device = d.pop("showOnSingleDevice", UNSET)

        show_on_multi_device = d.pop("showOnMultiDevice", UNSET)

        show_on_teleop = d.pop("showOnTeleop", UNSET)

        show_on_analytics = d.pop("showOnAnalytics", UNSET)

        show_timeline = d.pop("showTimeline", UNSET)

        _filter_ = d.pop("filter", UNSET)
        filter_: Union[Unset, Filter]
        if isinstance(_filter_, Unset):
            filter_ = UNSET
        else:
            filter_ = Filter.from_dict(_filter_)

        layout = d.pop("layout", UNSET)

        _layout_type = d.pop("layoutType", UNSET)
        layout_type: Union[Unset, None, PartialViewLayoutType]
        if _layout_type is None:
            layout_type = None
        elif isinstance(_layout_type, Unset):
            layout_type = UNSET
        else:
            layout_type = PartialViewLayoutType(_layout_type)

        configuration = []
        _configuration = d.pop("configuration", UNSET)
        for configuration_item_data in _configuration or []:
            configuration_item = ViewConfiguration.from_dict(configuration_item_data)

            configuration.append(configuration_item)

        index = d.pop("index", UNSET)

        id = d.pop("id", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        _tags = d.pop("tags", UNSET)
        tags: Union[Unset, PartialViewTags]
        if isinstance(_tags, Unset):
            tags = UNSET
        else:
            tags = PartialViewTags.from_dict(_tags)

        partial_view = cls(
            organization_id=organization_id,
            name=name,
            url=url,
            show_on_single_device=show_on_single_device,
            show_on_multi_device=show_on_multi_device,
            show_on_teleop=show_on_teleop,
            show_on_analytics=show_on_analytics,
            show_timeline=show_timeline,
            filter_=filter_,
            layout=layout,
            layout_type=layout_type,
            configuration=configuration,
            index=index,
            id=id,
            created_at=created_at,
            updated_at=updated_at,
            tags=tags,
        )

        partial_view.additional_properties = d
        return partial_view

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.annotation_field_type import AnnotationFieldType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.sheet_parameters import SheetParameters
    from ..models.tag_parameters import TagParameters
    from ..models.user_parameters import UserParameters


T = TypeVar("T", bound="AnnotationField")


@attr.s(auto_attribs=True)
class AnnotationField:
    """
    Attributes:
        type (AnnotationFieldType):
        key (Any):
        name (str):
        required (bool):
        parameters (Union['SheetParameters', 'TagParameters', 'UserParameters']):
        description (Union[Unset, str]):
    """

    type: AnnotationFieldType
    key: Any
    name: str
    required: bool
    parameters: Union["SheetParameters", "TagParameters", "UserParameters"]
    description: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.tag_parameters import TagParameters
        from ..models.user_parameters import UserParameters

        type = self.type.value

        key = self.key
        name = self.name
        required = self.required
        parameters: Dict[str, Any]

        if isinstance(self.parameters, TagParameters):
            parameters = self.parameters.to_dict()

        elif isinstance(self.parameters, UserParameters):
            parameters = self.parameters.to_dict()

        else:
            parameters = self.parameters.to_dict()

        description = self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "key": key,
                "name": name,
                "required": required,
                "parameters": parameters,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.sheet_parameters import SheetParameters
        from ..models.tag_parameters import TagParameters
        from ..models.user_parameters import UserParameters

        d = src_dict.copy()
        type = AnnotationFieldType(d.pop("type"))

        key = d.pop("key")

        name = d.pop("name")

        required = d.pop("required")

        def _parse_parameters(data: object) -> Union["SheetParameters", "TagParameters", "UserParameters"]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                parameters_type_0 = TagParameters.from_dict(data)

                return parameters_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                parameters_type_1 = UserParameters.from_dict(data)

                return parameters_type_1
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            parameters_type_2 = SheetParameters.from_dict(data)

            return parameters_type_2

        parameters = _parse_parameters(d.pop("parameters"))

        description = d.pop("description", UNSET)

        annotation_field = cls(
            type=type,
            key=key,
            name=name,
            required=required,
            parameters=parameters,
            description=description,
        )

        annotation_field.additional_properties = d
        return annotation_field

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import unittest
from formant.sdk.cloud.v2.src.admin_api import AdminAPI
from formant.sdk.cloud.v2.formant_admin_api_client import AuthenticatedClient
import os

EMAIL = os.getenv("FORMANT_EMAIL")
PASSWORD = os.getenv("FORMANT_PASSWORD")
BASE_URL = "https://api.formant.io/v1"


class TestAuthentication(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestAuthentication, self).__init__(*args, **kwargs)
        self.client = AdminAPI(email=EMAIL, password=PASSWORD, base_url=BASE_URL)

    def test_authenticate(self):
        new_client = self.client.get_client()
        self.assertIs(type(new_client), AuthenticatedClient)

    def test_authenticate_refresh(self):
        _ = self.client.get_client()
        new_client = self.client.get_client()
        self.assertIs(type(new_client), AuthenticatedClient)


unittest.main()



from formant.sdk.cloud.v2.formant_admin_api_client import Client, AuthenticatedClient
from formant.sdk.cloud.v2.formant_admin_api_client.models import (
    LoginRequest,
    RefreshRequest,
    LoginResult,
)
from formant.sdk.cloud.v2.formant_admin_api_client.api.auth import (
    auth_controller_login,
    auth_controller_refresh,
)
from formant.sdk.cloud.v2.formant_admin_api_client.types import Response
import time

TOKEN_EXPIRATION_SECONDS = 3600


class Auth:
    def __init__(
        self, email: str, password: str, api: str, base_url: str, timeout: int = 30
    ):
        self._email = email
        self._password = password
        self._timeout = timeout
        self._api = api
        self._base_url = base_url
        self._headers = {
            "Content-Type": "application/json",
            "App-ID": "formant/python-cloud-sdk",
        }
        self._unauthenticated_client = Client(base_url="%s/admin" % base_url)
        self._token_expiry = None
        self._access_token = None
        self._refresh_token = None
        self._user_id = None
        self.organization_id = None

    def get_client(self):
        hasRefreshToken = self._refresh_token is not None
        accessTokenIsNone = self._access_token is None
        tokenHasExpired = (
            True
            if self._token_expiry is None
            else self._token_expiry < int(time.time())
        )
        shouldGetNewAccessToken = accessTokenIsNone or tokenHasExpired
        if hasRefreshToken:
            self._refresh_existing_client()
        elif shouldGetNewAccessToken:
            self._create_new_client()

        return self.client

    def _create_new_client(self):
        login_request = LoginRequest(
            email=self._email,
            password=self._password,
            token_expiration_seconds=TOKEN_EXPIRATION_SECONDS,
        )

        response: Response[LoginResult] = auth_controller_login.sync(
            client=self._unauthenticated_client, json_body=login_request
        )
        wasAuthenticationSuccessful = response is not None
        if not wasAuthenticationSuccessful:
            raise ValueError("Authentication failed")
        result = response.authentication

        self._token_expiry = int(time.time()) + 3530
        self._access_token = result.access_token
        self._refresh_token = result.refresh_token
        self.organization_id = result.organization_id
        self._headers["Org-ID"] = self.organization_id
        self._user_id = result.user_id
        self.client = AuthenticatedClient(
            base_url="%s/%s" % (self._base_url, self._api),
            headers=self._headers,
            token=self._access_token,
        ).with_timeout(self._timeout)

    def _refresh_existing_client(self):
        refresh_request = RefreshRequest(
            self._refresh_token, token_expiration_seconds=TOKEN_EXPIRATION_SECONDS
        )
        response: Response[LoginResult] = auth_controller_refresh.sync(
            client=self._unauthenticated_client, json_body=refresh_request
        )
        result = response.authentication
        self._token_expiry = int(time.time()) + 3530
        self._access_token = result.access_token
        self.client = AuthenticatedClient(
            base_url=f"{self._base_url}/{self._api}", token=self._access_token
        )



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.annotation_field_value import AnnotationFieldValue


T = TypeVar("T", bound="GoogleSheetParseResult")


@attr.s(auto_attribs=True)
class GoogleSheetParseResult:
    """
    Attributes:
        header_invalid (Union[Unset, bool]):
        header_preview (Union[Unset, List[str]]):
        content_invalid (Union[Unset, bool]):
        values (Union[Unset, List['AnnotationFieldValue']]):
    """

    header_invalid: Union[Unset, bool] = UNSET
    header_preview: Union[Unset, List[str]] = UNSET
    content_invalid: Union[Unset, bool] = UNSET
    values: Union[Unset, List["AnnotationFieldValue"]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        header_invalid = self.header_invalid
        header_preview: Union[Unset, List[str]] = UNSET
        if not isinstance(self.header_preview, Unset):
            header_preview = self.header_preview

        content_invalid = self.content_invalid
        values: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.values, Unset):
            values = []
            for values_item_data in self.values:
                values_item = values_item_data.to_dict()

                values.append(values_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if header_invalid is not UNSET:
            field_dict["headerInvalid"] = header_invalid
        if header_preview is not UNSET:
            field_dict["headerPreview"] = header_preview
        if content_invalid is not UNSET:
            field_dict["contentInvalid"] = content_invalid
        if values is not UNSET:
            field_dict["values"] = values

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.annotation_field_value import AnnotationFieldValue

        d = src_dict.copy()
        header_invalid = d.pop("headerInvalid", UNSET)

        header_preview = cast(List[str], d.pop("headerPreview", UNSET))

        content_invalid = d.pop("contentInvalid", UNSET)

        values = []
        _values = d.pop("values", UNSET)
        for values_item_data in _values or []:
            values_item = AnnotationFieldValue.from_dict(values_item_data)

            values.append(values_item)

        google_sheet_parse_result = cls(
            header_invalid=header_invalid,
            header_preview=header_preview,
            content_invalid=content_invalid,
            values=values,
        )

        google_sheet_parse_result.additional_properties = d
        return google_sheet_parse_result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import unittest
from formant.sdk.cloud.v2.src.admin_api import AdminAPI
from formant.sdk.cloud.v2.formant_admin_api_client.models import EventQuery
from datetime import datetime
import os

EMAIL = os.getenv("FORMANT_EMAIL")
PASSWORD = os.getenv("FORMANT_PASSWORD")
BASE_URL = "https://api.formant.io/v1"


class TestEvents(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestEvents, self).__init__(*args, **kwargs)
        self.client = AdminAPI(email=EMAIL, password=PASSWORD, base_url=BASE_URL)

    def test_query(self):
        event_query = EventQuery(count=3)
        result = self.client.events.query(event_query)
        self.assertEqual(result.status_code, 200)


unittest.main()



import datetime
from typing import Any, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.schedule_type import ScheduleType
from ..types import UNSET, Unset

T = TypeVar("T", bound="Schedule")


@attr.s(auto_attribs=True)
class Schedule:
    """
    Attributes:
        name (str):
        description (str):
        type (ScheduleType):
        duration_ms (int):
        timezone (str):
        device_id (str):
        status (str):
        organization_id (Union[Unset, str]):
        at (Optional[datetime.datetime]):
        cron (Optional[str]):
        command_template_id (Optional[str]):
        parameter_value (Optional[str]):
        id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
    """

    name: str
    description: str
    type: ScheduleType
    duration_ms: int
    timezone: str
    device_id: str
    status: str
    at: Optional[datetime.datetime]
    cron: Optional[str]
    command_template_id: Optional[str]
    parameter_value: Optional[str]
    organization_id: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        description = self.description
        type = self.type.value

        duration_ms = self.duration_ms
        timezone = self.timezone
        device_id = self.device_id
        status = self.status
        organization_id = self.organization_id
        at = self.at.isoformat() if self.at else None

        cron = self.cron
        command_template_id = self.command_template_id
        parameter_value = self.parameter_value
        id = self.id
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "description": description,
                "type": type,
                "durationMs": duration_ms,
                "timezone": timezone,
                "deviceId": device_id,
                "status": status,
                "at": at,
                "cron": cron,
                "commandTemplateId": command_template_id,
                "parameterValue": parameter_value,
            }
        )
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id
        if id is not UNSET:
            field_dict["id"] = id
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        description = d.pop("description")

        type = ScheduleType(d.pop("type"))

        duration_ms = d.pop("durationMs")

        timezone = d.pop("timezone")

        device_id = d.pop("deviceId")

        status = d.pop("status")

        organization_id = d.pop("organizationId", UNSET)

        _at = d.pop("at")
        at: Optional[datetime.datetime]
        if _at is None:
            at = None
        else:
            at = isoparse(_at)

        cron = d.pop("cron")

        command_template_id = d.pop("commandTemplateId")

        parameter_value = d.pop("parameterValue")

        id = d.pop("id", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        schedule = cls(
            name=name,
            description=description,
            type=type,
            duration_ms=duration_ms,
            timezone=timezone,
            device_id=device_id,
            status=status,
            organization_id=organization_id,
            at=at,
            cron=cron,
            command_template_id=command_template_id,
            parameter_value=parameter_value,
            id=id,
            created_at=created_at,
            updated_at=updated_at,
        )

        schedule.additional_properties = d
        return schedule

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties




from formant.sdk.cloud.v2.formant_admin_api_client.models import Filter, LocationViewport, ViewTags, LocationViewConfigurationBasemap, PartialViewTags, Vector3, ImageViewConfiguration, Camera, ViewLayoutType, Location, ViewListResponse, LocationViewConfiguration, ViewConfiguration, View, LocalizationViewConfiguration, PointCloudViewConfiguration, BitsetViewConfiguration, NumericViewConfiguration, PartialViewLayoutType, TransformTreeViewConfiguration, ViewConfigurationType, ImageViewConfigurationMode, PartialView
from formant.sdk.cloud.v2.formant_admin_api_client.api.view import view_controller_get_one, view_controller_get_all, view_controller_patch
from formant.sdk.cloud.v2.formant_admin_api_client.types import *



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.adapter_configuration import AdapterConfiguration
    from ..models.device_application_configuration import DeviceApplicationConfiguration
    from ..models.device_blob_data import DeviceBlobData
    from ..models.device_configuration_document_tags import DeviceConfigurationDocumentTags
    from ..models.device_diagnostics_configuration import DeviceDiagnosticsConfiguration
    from ..models.device_port_forwarding_configuration import DevicePortForwardingConfiguration
    from ..models.device_resources_configuration import DeviceResourcesConfiguration
    from ..models.device_telemetry_configuration import DeviceTelemetryConfiguration
    from ..models.device_teleop_configuration import DeviceTeleopConfiguration
    from ..models.rtc_info import RtcInfo


T = TypeVar("T", bound="DeviceConfigurationDocument")


@attr.s(auto_attribs=True)
class DeviceConfigurationDocument:
    """
    Attributes:
        tags (Union[Unset, DeviceConfigurationDocumentTags]):
        resources (Union[Unset, DeviceResourcesConfiguration]):
        telemetry (Union[Unset, DeviceTelemetryConfiguration]):
        application (Union[Unset, DeviceApplicationConfiguration]):
        blob_data (Union[Unset, DeviceBlobData]):
        terminal_access (Union[Unset, None, bool]):
        teleop (Union[Unset, DeviceTeleopConfiguration]):
        port_forwarding (Union[Unset, DevicePortForwardingConfiguration]):
        diagnostics (Union[Unset, DeviceDiagnosticsConfiguration]):
        urdf_files (Union[Unset, List[str]]):
        adapters (Union[Unset, List['AdapterConfiguration']]):
        rtc_info (Union[Unset, RtcInfo]):
    """

    tags: Union[Unset, "DeviceConfigurationDocumentTags"] = UNSET
    resources: Union[Unset, "DeviceResourcesConfiguration"] = UNSET
    telemetry: Union[Unset, "DeviceTelemetryConfiguration"] = UNSET
    application: Union[Unset, "DeviceApplicationConfiguration"] = UNSET
    blob_data: Union[Unset, "DeviceBlobData"] = UNSET
    terminal_access: Union[Unset, None, bool] = UNSET
    teleop: Union[Unset, "DeviceTeleopConfiguration"] = UNSET
    port_forwarding: Union[Unset, "DevicePortForwardingConfiguration"] = UNSET
    diagnostics: Union[Unset, "DeviceDiagnosticsConfiguration"] = UNSET
    urdf_files: Union[Unset, List[str]] = UNSET
    adapters: Union[Unset, List["AdapterConfiguration"]] = UNSET
    rtc_info: Union[Unset, "RtcInfo"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        tags: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags.to_dict()

        resources: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.resources, Unset):
            resources = self.resources.to_dict()

        telemetry: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.telemetry, Unset):
            telemetry = self.telemetry.to_dict()

        application: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.application, Unset):
            application = self.application.to_dict()

        blob_data: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.blob_data, Unset):
            blob_data = self.blob_data.to_dict()

        terminal_access = self.terminal_access
        teleop: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.teleop, Unset):
            teleop = self.teleop.to_dict()

        port_forwarding: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.port_forwarding, Unset):
            port_forwarding = self.port_forwarding.to_dict()

        diagnostics: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.diagnostics, Unset):
            diagnostics = self.diagnostics.to_dict()

        urdf_files: Union[Unset, List[str]] = UNSET
        if not isinstance(self.urdf_files, Unset):
            urdf_files = self.urdf_files

        adapters: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.adapters, Unset):
            adapters = []
            for adapters_item_data in self.adapters:
                adapters_item = adapters_item_data.to_dict()

                adapters.append(adapters_item)

        rtc_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.rtc_info, Unset):
            rtc_info = self.rtc_info.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if tags is not UNSET:
            field_dict["tags"] = tags
        if resources is not UNSET:
            field_dict["resources"] = resources
        if telemetry is not UNSET:
            field_dict["telemetry"] = telemetry
        if application is not UNSET:
            field_dict["application"] = application
        if blob_data is not UNSET:
            field_dict["blobData"] = blob_data
        if terminal_access is not UNSET:
            field_dict["terminalAccess"] = terminal_access
        if teleop is not UNSET:
            field_dict["teleop"] = teleop
        if port_forwarding is not UNSET:
            field_dict["portForwarding"] = port_forwarding
        if diagnostics is not UNSET:
            field_dict["diagnostics"] = diagnostics
        if urdf_files is not UNSET:
            field_dict["urdfFiles"] = urdf_files
        if adapters is not UNSET:
            field_dict["adapters"] = adapters
        if rtc_info is not UNSET:
            field_dict["rtcInfo"] = rtc_info

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.adapter_configuration import AdapterConfiguration
        from ..models.device_application_configuration import DeviceApplicationConfiguration
        from ..models.device_blob_data import DeviceBlobData
        from ..models.device_configuration_document_tags import DeviceConfigurationDocumentTags
        from ..models.device_diagnostics_configuration import DeviceDiagnosticsConfiguration
        from ..models.device_port_forwarding_configuration import DevicePortForwardingConfiguration
        from ..models.device_resources_configuration import DeviceResourcesConfiguration
        from ..models.device_telemetry_configuration import DeviceTelemetryConfiguration
        from ..models.device_teleop_configuration import DeviceTeleopConfiguration
        from ..models.rtc_info import RtcInfo

        d = src_dict.copy()
        _tags = d.pop("tags", UNSET)
        tags: Union[Unset, DeviceConfigurationDocumentTags]
        if isinstance(_tags, Unset):
            tags = UNSET
        else:
            tags = DeviceConfigurationDocumentTags.from_dict(_tags)

        _resources = d.pop("resources", UNSET)
        resources: Union[Unset, DeviceResourcesConfiguration]
        if isinstance(_resources, Unset):
            resources = UNSET
        else:
            resources = DeviceResourcesConfiguration.from_dict(_resources)

        _telemetry = d.pop("telemetry", UNSET)
        telemetry: Union[Unset, DeviceTelemetryConfiguration]
        if isinstance(_telemetry, Unset):
            telemetry = UNSET
        else:
            telemetry = DeviceTelemetryConfiguration.from_dict(_telemetry)

        _application = d.pop("application", UNSET)
        application: Union[Unset, DeviceApplicationConfiguration]
        if isinstance(_application, Unset):
            application = UNSET
        else:
            application = DeviceApplicationConfiguration.from_dict(_application)

        _blob_data = d.pop("blobData", UNSET)
        blob_data: Union[Unset, DeviceBlobData]
        if isinstance(_blob_data, Unset):
            blob_data = UNSET
        else:
            blob_data = DeviceBlobData.from_dict(_blob_data)

        terminal_access = d.pop("terminalAccess", UNSET)

        _teleop = d.pop("teleop", UNSET)
        teleop: Union[Unset, DeviceTeleopConfiguration]
        if isinstance(_teleop, Unset):
            teleop = UNSET
        else:
            teleop = DeviceTeleopConfiguration.from_dict(_teleop)

        _port_forwarding = d.pop("portForwarding", UNSET)
        port_forwarding: Union[Unset, DevicePortForwardingConfiguration]
        if isinstance(_port_forwarding, Unset):
            port_forwarding = UNSET
        else:
            port_forwarding = DevicePortForwardingConfiguration.from_dict(_port_forwarding)

        _diagnostics = d.pop("diagnostics", UNSET)
        diagnostics: Union[Unset, DeviceDiagnosticsConfiguration]
        if isinstance(_diagnostics, Unset):
            diagnostics = UNSET
        else:
            diagnostics = DeviceDiagnosticsConfiguration.from_dict(_diagnostics)

        urdf_files = cast(List[str], d.pop("urdfFiles", UNSET))

        adapters = []
        _adapters = d.pop("adapters", UNSET)
        for adapters_item_data in _adapters or []:
            adapters_item = AdapterConfiguration.from_dict(adapters_item_data)

            adapters.append(adapters_item)

        _rtc_info = d.pop("rtcInfo", UNSET)
        rtc_info: Union[Unset, RtcInfo]
        if isinstance(_rtc_info, Unset):
            rtc_info = UNSET
        else:
            rtc_info = RtcInfo.from_dict(_rtc_info)

        device_configuration_document = cls(
            tags=tags,
            resources=resources,
            telemetry=telemetry,
            application=application,
            blob_data=blob_data,
            terminal_access=terminal_access,
            teleop=teleop,
            port_forwarding=port_forwarding,
            diagnostics=diagnostics,
            urdf_files=urdf_files,
            adapters=adapters,
            rtc_info=rtc_info,
        )

        device_configuration_document.additional_properties = d
        return device_configuration_document

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.goal import Goal
    from ..models.map_ import Map
    from ..models.odometry import Odometry
    from ..models.path import Path
    from ..models.point_cloud import PointCloud


T = TypeVar("T", bound="Localization")


@attr.s(auto_attribs=True)
class Localization:
    """
    Attributes:
        odometry (Union[Unset, Odometry]):
        map_ (Union[Unset, Map]):
        point_clouds (Union[Unset, List['PointCloud']]):
        path (Union[Unset, Path]):
        goal (Union[Unset, Goal]):
        url (Union[Unset, str]):
        size (Union[Unset, int]):
    """

    odometry: Union[Unset, "Odometry"] = UNSET
    map_: Union[Unset, "Map"] = UNSET
    point_clouds: Union[Unset, List["PointCloud"]] = UNSET
    path: Union[Unset, "Path"] = UNSET
    goal: Union[Unset, "Goal"] = UNSET
    url: Union[Unset, str] = UNSET
    size: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        odometry: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.odometry, Unset):
            odometry = self.odometry.to_dict()

        map_: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.map_, Unset):
            map_ = self.map_.to_dict()

        point_clouds: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.point_clouds, Unset):
            point_clouds = []
            for point_clouds_item_data in self.point_clouds:
                point_clouds_item = point_clouds_item_data.to_dict()

                point_clouds.append(point_clouds_item)

        path: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.path, Unset):
            path = self.path.to_dict()

        goal: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.goal, Unset):
            goal = self.goal.to_dict()

        url = self.url
        size = self.size

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if odometry is not UNSET:
            field_dict["odometry"] = odometry
        if map_ is not UNSET:
            field_dict["map"] = map_
        if point_clouds is not UNSET:
            field_dict["pointClouds"] = point_clouds
        if path is not UNSET:
            field_dict["path"] = path
        if goal is not UNSET:
            field_dict["goal"] = goal
        if url is not UNSET:
            field_dict["url"] = url
        if size is not UNSET:
            field_dict["size"] = size

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.goal import Goal
        from ..models.map_ import Map
        from ..models.odometry import Odometry
        from ..models.path import Path
        from ..models.point_cloud import PointCloud

        d = src_dict.copy()
        _odometry = d.pop("odometry", UNSET)
        odometry: Union[Unset, Odometry]
        if isinstance(_odometry, Unset):
            odometry = UNSET
        else:
            odometry = Odometry.from_dict(_odometry)

        _map_ = d.pop("map", UNSET)
        map_: Union[Unset, Map]
        if isinstance(_map_, Unset):
            map_ = UNSET
        else:
            map_ = Map.from_dict(_map_)

        point_clouds = []
        _point_clouds = d.pop("pointClouds", UNSET)
        for point_clouds_item_data in _point_clouds or []:
            point_clouds_item = PointCloud.from_dict(point_clouds_item_data)

            point_clouds.append(point_clouds_item)

        _path = d.pop("path", UNSET)
        path: Union[Unset, Path]
        if isinstance(_path, Unset):
            path = UNSET
        else:
            path = Path.from_dict(_path)

        _goal = d.pop("goal", UNSET)
        goal: Union[Unset, Goal]
        if isinstance(_goal, Unset):
            goal = UNSET
        else:
            goal = Goal.from_dict(_goal)

        url = d.pop("url", UNSET)

        size = d.pop("size", UNSET)

        localization = cls(
            odometry=odometry,
            map_=map_,
            point_clouds=point_clouds,
            path=path,
            goal=goal,
            url=url,
            size=size,
        )

        localization.additional_properties = d
        return localization

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Optional, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="AnalyticsModule")


@attr.s(auto_attribs=True)
class AnalyticsModule:
    """
    Attributes:
        name (str):
        query (Any):
        layout (Any):
        stream_ids (List[str]):
        organization_id (Union[Unset, str]):
        data (Optional[List[Any]]):
        fullscreen (Union[Unset, bool]):
        id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
    """

    name: str
    query: Any
    layout: Any
    stream_ids: List[str]
    data: Optional[List[Any]]
    organization_id: Union[Unset, str] = UNSET
    fullscreen: Union[Unset, bool] = UNSET
    id: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        query = self.query
        layout = self.layout
        stream_ids = self.stream_ids

        organization_id = self.organization_id
        if self.data is None:
            data = None
        else:
            data = self.data

        fullscreen = self.fullscreen
        id = self.id
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "query": query,
                "layout": layout,
                "streamIds": stream_ids,
                "data": data,
            }
        )
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id
        if fullscreen is not UNSET:
            field_dict["fullscreen"] = fullscreen
        if id is not UNSET:
            field_dict["id"] = id
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        query = d.pop("query")

        layout = d.pop("layout")

        stream_ids = cast(List[str], d.pop("streamIds"))

        organization_id = d.pop("organizationId", UNSET)

        data = cast(List[Any], d.pop("data"))

        fullscreen = d.pop("fullscreen", UNSET)

        id = d.pop("id", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        analytics_module = cls(
            name=name,
            query=query,
            layout=layout,
            stream_ids=stream_ids,
            organization_id=organization_id,
            data=data,
            fullscreen=fullscreen,
            id=id,
            created_at=created_at,
            updated_at=updated_at,
        )

        analytics_module.additional_properties = d
        return analytics_module

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="EventCounts")


@attr.s(auto_attribs=True)
class EventCounts:
    """
    Attributes:
        total (int):
        info (Union[Unset, int]):
        warn (Union[Unset, int]):
        error (Union[Unset, int]):
        critical (Union[Unset, int]):
        triggered_events (Union[Unset, int]):
        intervention_request (Union[Unset, int]):
    """

    total: int
    info: Union[Unset, int] = UNSET
    warn: Union[Unset, int] = UNSET
    error: Union[Unset, int] = UNSET
    critical: Union[Unset, int] = UNSET
    triggered_events: Union[Unset, int] = UNSET
    intervention_request: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        total = self.total
        info = self.info
        warn = self.warn
        error = self.error
        critical = self.critical
        triggered_events = self.triggered_events
        intervention_request = self.intervention_request

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "total": total,
            }
        )
        if info is not UNSET:
            field_dict["info"] = info
        if warn is not UNSET:
            field_dict["warn"] = warn
        if error is not UNSET:
            field_dict["error"] = error
        if critical is not UNSET:
            field_dict["critical"] = critical
        if triggered_events is not UNSET:
            field_dict["triggered-events"] = triggered_events
        if intervention_request is not UNSET:
            field_dict["intervention-request"] = intervention_request

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        total = d.pop("total")

        info = d.pop("info", UNSET)

        warn = d.pop("warn", UNSET)

        error = d.pop("error", UNSET)

        critical = d.pop("critical", UNSET)

        triggered_events = d.pop("triggered-events", UNSET)

        intervention_request = d.pop("intervention-request", UNSET)

        event_counts = cls(
            total=total,
            info=info,
            warn=warn,
            error=error,
            critical=critical,
            triggered_events=triggered_events,
            intervention_request=intervention_request,
        )

        event_counts.additional_properties = d
        return event_counts

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.vector_3 import Vector3


T = TypeVar("T", bound="Twist")


@attr.s(auto_attribs=True)
class Twist:
    """
    Attributes:
        linear (Vector3):
        angular (Vector3):
    """

    linear: "Vector3"
    angular: "Vector3"
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        linear = self.linear.to_dict()

        angular = self.angular.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "linear": linear,
                "angular": angular,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.vector_3 import Vector3

        d = src_dict.copy()
        linear = Vector3.from_dict(d.pop("linear"))

        angular = Vector3.from_dict(d.pop("angular"))

        twist = cls(
            linear=linear,
            angular=angular,
        )

        twist.additional_properties = d
        return twist

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.on_demand_presence_stream_item_group import OnDemandPresenceStreamItemGroup


T = TypeVar("T", bound="OnDemandStreamPresence")


@attr.s(auto_attribs=True)
class OnDemandStreamPresence:
    """
    Attributes:
        stream_name (str):
        presence (List['OnDemandPresenceStreamItemGroup']):
    """

    stream_name: str
    presence: List["OnDemandPresenceStreamItemGroup"]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        stream_name = self.stream_name
        presence = []
        for presence_item_data in self.presence:
            presence_item = presence_item_data.to_dict()

            presence.append(presence_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "streamName": stream_name,
                "presence": presence,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.on_demand_presence_stream_item_group import OnDemandPresenceStreamItemGroup

        d = src_dict.copy()
        stream_name = d.pop("streamName")

        presence = []
        _presence = d.pop("presence")
        for presence_item_data in _presence:
            presence_item = OnDemandPresenceStreamItemGroup.from_dict(presence_item_data)

            presence.append(presence_item)

        on_demand_stream_presence = cls(
            stream_name=stream_name,
            presence=presence,
        )

        on_demand_stream_presence.additional_properties = d
        return on_demand_stream_presence

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="TeleopViewConfiguration")


@attr.s(auto_attribs=True)
class TeleopViewConfiguration:
    """
    Attributes:
        stream_name (Union[Unset, Any]):
        configuration_id (Union[Unset, Any]):
    """

    stream_name: Union[Unset, Any] = UNSET
    configuration_id: Union[Unset, Any] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        stream_name = self.stream_name
        configuration_id = self.configuration_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if stream_name is not UNSET:
            field_dict["streamName"] = stream_name
        if configuration_id is not UNSET:
            field_dict["configurationId"] = configuration_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        stream_name = d.pop("streamName", UNSET)

        configuration_id = d.pop("configurationId", UNSET)

        teleop_view_configuration = cls(
            stream_name=stream_name,
            configuration_id=configuration_id,
        )

        teleop_view_configuration.additional_properties = d
        return teleop_view_configuration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.slack_webhook import SlackWebhook


T = TypeVar("T", bound="SlackInfo")


@attr.s(auto_attribs=True)
class SlackInfo:
    """
    Attributes:
        webhooks (Union[Unset, List['SlackWebhook']]):
        webhook_url (Union[Unset, str]):
        webhook_channel (Union[Unset, str]):
        configuration_url (Union[Unset, str]):
        team_id (Union[Unset, str]):
        token (Union[Unset, str]):
    """

    webhooks: Union[Unset, List["SlackWebhook"]] = UNSET
    webhook_url: Union[Unset, str] = UNSET
    webhook_channel: Union[Unset, str] = UNSET
    configuration_url: Union[Unset, str] = UNSET
    team_id: Union[Unset, str] = UNSET
    token: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        webhooks: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.webhooks, Unset):
            webhooks = []
            for webhooks_item_data in self.webhooks:
                webhooks_item = webhooks_item_data.to_dict()

                webhooks.append(webhooks_item)

        webhook_url = self.webhook_url
        webhook_channel = self.webhook_channel
        configuration_url = self.configuration_url
        team_id = self.team_id
        token = self.token

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if webhooks is not UNSET:
            field_dict["webhooks"] = webhooks
        if webhook_url is not UNSET:
            field_dict["webhookUrl"] = webhook_url
        if webhook_channel is not UNSET:
            field_dict["webhookChannel"] = webhook_channel
        if configuration_url is not UNSET:
            field_dict["configurationUrl"] = configuration_url
        if team_id is not UNSET:
            field_dict["teamId"] = team_id
        if token is not UNSET:
            field_dict["token"] = token

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.slack_webhook import SlackWebhook

        d = src_dict.copy()
        webhooks = []
        _webhooks = d.pop("webhooks", UNSET)
        for webhooks_item_data in _webhooks or []:
            webhooks_item = SlackWebhook.from_dict(webhooks_item_data)

            webhooks.append(webhooks_item)

        webhook_url = d.pop("webhookUrl", UNSET)

        webhook_channel = d.pop("webhookChannel", UNSET)

        configuration_url = d.pop("configurationUrl", UNSET)

        team_id = d.pop("teamId", UNSET)

        token = d.pop("token", UNSET)

        slack_info = cls(
            webhooks=webhooks,
            webhook_url=webhook_url,
            webhook_channel=webhook_channel,
            configuration_url=configuration_url,
            team_id=team_id,
            token=token,
        )

        slack_info.additional_properties = d
        return slack_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.count_history_entry_count import CountHistoryEntryCount


T = TypeVar("T", bound="CountHistoryEntry")


@attr.s(auto_attribs=True)
class CountHistoryEntry:
    """
    Attributes:
        id (str):
        counts (List['CountHistoryEntryCount']):
    """

    id: str
    counts: List["CountHistoryEntryCount"]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        counts = []
        for counts_item_data in self.counts:
            counts_item = counts_item_data.to_dict()

            counts.append(counts_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "counts": counts,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.count_history_entry_count import CountHistoryEntryCount

        d = src_dict.copy()
        id = d.pop("id")

        counts = []
        _counts = d.pop("counts")
        for counts_item_data in _counts:
            counts_item = CountHistoryEntryCount.from_dict(counts_item_data)

            counts.append(counts_item)

        count_history_entry = cls(
            id=id,
            counts=counts,
        )

        count_history_entry.additional_properties = d
        return count_history_entry

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties




from formant.sdk.cloud.v2.formant_admin_api_client.models import DevicePortForwardingConfiguration, DeviceConfigurationTemplateListResponse, DeviceTelemetryConfiguration, OnvifDevice, DeviceOfflineEventStreamType, RosTopic, DeviceTeleopHardwareStreamConfigurationRtcStreamType, DeviceStreamDirectoryWatchConfiguration, DeviceListResponse, CommandProgress, DeviceOnlineEventSeverity, TeleopJoystickAxisConfigurationDimension, AdapterConfiguration, DeviceStreamFileTailConfiguration, OnDemandPresenceStreamItemGroupDatapointType, DeviceTeleopHardwareStreamConfiguration, DeviceReportedConfigurationState, DeviceStreamConfigurationTags, DeviceApplicationConfiguration, DeviceStreamDirectoryWatchConfigurationType, DeviceTeleopHardwareStreamConfigurationQuality, DeviceTeleopConfiguration, TeleopJoystickConfigurationPosition, OnDemandBufferBufferType, DeviceDetailsListResponse, Network, TeleopViewConfiguration, DeviceStreamRosTopicConfiguration, DeviceOfflineEvent, DeviceTeleopRosStreamConfigurationTopicType, RtcInfoRtcIceTransportPoliciesItem, DeviceTeleopRosStreamConfigurationMode, DeviceStreamRosLocalizationConfiguration, DeviceQueryType, DeviceStreamTransformConfiguration, TeleopJoystickAxisConfiguration, DeviceTeleopRosStreamConfiguration, DeviceStreamCustomConfigurationType, Device, DeviceDiskConfiguration, DeviceConfigurationTemplate, DeviceApplicationConfigurationConfigurationMap, DeviceFollower, DeviceStreamRosTopicConfigurationType, DeviceType, DeviceProvisioningRequest, DeviceStreamRosLocalizationConfigurationType, DeviceResourcesConfiguration, KernelInfo, ScopeFilter, DeviceTags, DeviceTeleopCustomStreamConfigurationMode, DeviceState, OsInfo, PartialDeviceType, DeviceStreamCustomConfiguration, RtcInfoRtcIceServerProtocol, DeviceDetailsTags, DeviceStreamFileTailConfigurationType, DeviceStateEnv, DeviceConfiguration, AudioDevice, DeviceStreamDirectoryWatchConfigurationFileType, PartialDeviceTags, DeviceBlobData, HwInfo, DeviceTeleopHardwareStreamConfigurationHardwareType, DeviceTeleopCustomStreamConfigurationNumericControlVisualization, DeviceOfflineEventType, DeviceOnlineEvent, RtcInfo, ScopeFilterTypesItem, TeleopJoystickConfiguration, DeviceTeleopRosStreamConfigurationQuality, DeviceDetailsType, DeviceTeleopCustomStreamConfigurationQuality, OnDemandStreamPresence, OnDemandPresenceStreamItemGroup, DeviceDiagnosticsConfiguration, DeviceConfigurationDocument, DeviceStreamHardwareConfiguration, DeviceConfigurationDocumentTags, DeviceTeleopCustomStreamConfiguration, DeviceOnlineEventStreamType, DeviceOnlineEventTags, DeviceTeleopRosStreamConfigurationNumericControlVisualization, DeviceTeleopCustomStreamConfigurationRtcStreamType, DeviceStreamHardwareConfigurationType, DeviceStreamHardwareConfigurationHardwareType, DeviceOfflineEventTags, DeviceStreamFileTailConfigurationFileFormat, DeviceStreamRosTransformTreeConfiguration, OnDemandPresenceTimeRange, DeviceProvisioning, DeviceStreamRosTransformTreeConfigurationType, DeviceOnlineEventType, DeviceTeleopHardwareStreamConfigurationMode, NodeInfo, VideoDevice, DeviceStreamConfiguration, DeviceStreamHardwareConfigurationQuality, OnDemandBuffer, PartialDevice, DeviceQuery, DeviceRosConfiguration, OnDemandState, DeviceStreamConfigurationQuality, DeviceOfflineEventSeverity, DeviceConfigurationTemplateTags, DeviceDetails, NetworkInfo, DeviceRosState, DeviceTeleopRosStreamConfigurationAudioCodec, ValidationConfiguration, DeviceCredentials
from formant.sdk.cloud.v2.formant_admin_api_client.api.device import device_controller_query, device_controller_patch, device_controller_get_one, device_controller_get_configuration, device_controller_post_configuration
from formant.sdk.cloud.v2.formant_admin_api_client.types import *
from formant.sdk.cloud.v2.formant_admin_api_client.types import Optional




from formant.sdk.cloud.v2.formant_admin_api_client.models import CommandDeliveryEventStreamType, CommandQuery, FileInfo, CommandDeliveryEvent, CommandStreamType, CommandResponseStreamType, CommandProgress, CommandListResponse, CommandTemplateTags, CommandDeliveryEventTags, CommandTags, CommandDeliveryEventType, CommandParameterMeta, CommandParameter, CommandTemplateParameterMeta, CommandResponseTags, CommandResponse, Command, CommandRequestListResponse, CommandRequest, CommandTemplate, CommandTemplateListResponse
from formant.sdk.cloud.v2.formant_admin_api_client.api.command import command_controller_post, command_controller_query
from formant.sdk.cloud.v2.formant_admin_api_client.types import *



from formant.sdk.cloud.v2.src.auth import Auth
from formant.sdk.cloud.v2.src.resources.devices.devices import Devices
from formant.sdk.cloud.v2.src.resources.commands.commands import Commands
from formant.sdk.cloud.v2.src.resources.views.views import Views
from formant.sdk.cloud.v2.src.resources.annotations.annotations import Annotations
from formant.sdk.cloud.v2.src.resources.events.events import Events
from formant.sdk.cloud.v2.src.resources.files.files import Files
from formant.sdk.cloud.v2.src.resources.adapters.adapters import Adapters


class AdminAPI(Auth):
    def __init__(self, email: str, password: str, base_url: str, timeout: int = 30):
        super().__init__(
            email=email,
            password=password,
            api="admin",
            base_url=base_url,
            timeout=timeout,
        )
        self.devices = Devices(self.get_client)
        self.commands = Commands(self.get_client)
        self.views = Views(self.get_client)
        self.annotations = Annotations(self.get_client)
        self.events = Events(self.get_client)
        self.files = Files(self.get_client)
        self.adapters = Adapters(self.get_client)

        self.client = self.get_client()



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

from ..models.on_demand_presence_stream_item_group_datapoint_type import OnDemandPresenceStreamItemGroupDatapointType

if TYPE_CHECKING:
    from ..models.on_demand_presence_time_range import OnDemandPresenceTimeRange


T = TypeVar("T", bound="OnDemandPresenceStreamItemGroup")


@attr.s(auto_attribs=True)
class OnDemandPresenceStreamItemGroup:
    """
    Attributes:
        datapoint_type (OnDemandPresenceStreamItemGroupDatapointType):
        time_ranges (List['OnDemandPresenceTimeRange']):
    """

    datapoint_type: OnDemandPresenceStreamItemGroupDatapointType
    time_ranges: List["OnDemandPresenceTimeRange"]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        datapoint_type = self.datapoint_type.value

        time_ranges = []
        for time_ranges_item_data in self.time_ranges:
            time_ranges_item = time_ranges_item_data.to_dict()

            time_ranges.append(time_ranges_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "datapointType": datapoint_type,
                "timeRanges": time_ranges,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.on_demand_presence_time_range import OnDemandPresenceTimeRange

        d = src_dict.copy()
        datapoint_type = OnDemandPresenceStreamItemGroupDatapointType(d.pop("datapointType"))

        time_ranges = []
        _time_ranges = d.pop("timeRanges")
        for time_ranges_item_data in _time_ranges:
            time_ranges_item = OnDemandPresenceTimeRange.from_dict(time_ranges_item_data)

            time_ranges.append(time_ranges_item)

        on_demand_presence_stream_item_group = cls(
            datapoint_type=datapoint_type,
            time_ranges=time_ranges,
        )

        on_demand_presence_stream_item_group.additional_properties = d
        return on_demand_presence_stream_item_group

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

import attr

if TYPE_CHECKING:
    from ..models.count_history_entry import CountHistoryEntry


T = TypeVar("T", bound="CountHistory")


@attr.s(auto_attribs=True)
class CountHistory:
    """
    Attributes:
        total (int):
        counts (List['CountHistoryEntry']):
    """

    total: int
    counts: List["CountHistoryEntry"]
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        total = self.total
        counts = []
        for counts_item_data in self.counts:
            counts_item = counts_item_data.to_dict()

            counts.append(counts_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "total": total,
                "counts": counts,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.count_history_entry import CountHistoryEntry

        d = src_dict.copy()
        total = d.pop("total")

        counts = []
        _counts = d.pop("counts")
        for counts_item_data in _counts:
            counts_item = CountHistoryEntry.from_dict(counts_item_data)

            counts.append(counts_item)

        count_history = cls(
            total=total,
            counts=counts,
        )

        count_history.additional_properties = d
        return count_history

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.sql_query_types_item import SqlQueryTypesItem
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.stream_column import StreamColumn


T = TypeVar("T", bound="SqlQuery")


@attr.s(auto_attribs=True)
class SqlQuery:
    """
    Attributes:
        parameters (List[str]):
        sql_query (Union[Unset, str]):
        start (Union[Unset, datetime.datetime]):
        end (Union[Unset, datetime.datetime]):
        limit (Union[Unset, float]):
        stream_columns (Union[Unset, List['StreamColumn']]):
        select_columns (Union[Unset, List[str]]):
        order_by_column (Union[Unset, str]):
        order_by_descending (Union[Unset, bool]):
        filters (Union[Unset, List[str]]):
        next_ (Union[Unset, float]):
        agent_ids (Union[Unset, List[str]]):
        device_ids (Union[Unset, List[str]]):
        names (Union[Unset, List[str]]):
        types (Union[Unset, List[SqlQueryTypesItem]]):
        tags (Union[Unset, Any]):
        not_names (Union[Unset, List[str]]):
    """

    parameters: List[str]
    sql_query: Union[Unset, str] = UNSET
    start: Union[Unset, datetime.datetime] = UNSET
    end: Union[Unset, datetime.datetime] = UNSET
    limit: Union[Unset, float] = UNSET
    stream_columns: Union[Unset, List["StreamColumn"]] = UNSET
    select_columns: Union[Unset, List[str]] = UNSET
    order_by_column: Union[Unset, str] = UNSET
    order_by_descending: Union[Unset, bool] = UNSET
    filters: Union[Unset, List[str]] = UNSET
    next_: Union[Unset, float] = UNSET
    agent_ids: Union[Unset, List[str]] = UNSET
    device_ids: Union[Unset, List[str]] = UNSET
    names: Union[Unset, List[str]] = UNSET
    types: Union[Unset, List[SqlQueryTypesItem]] = UNSET
    tags: Union[Unset, Any] = UNSET
    not_names: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        parameters = self.parameters

        sql_query = self.sql_query
        start: Union[Unset, str] = UNSET
        if not isinstance(self.start, Unset):
            start = self.start.isoformat()

        end: Union[Unset, str] = UNSET
        if not isinstance(self.end, Unset):
            end = self.end.isoformat()

        limit = self.limit
        stream_columns: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.stream_columns, Unset):
            stream_columns = []
            for stream_columns_item_data in self.stream_columns:
                stream_columns_item = stream_columns_item_data.to_dict()

                stream_columns.append(stream_columns_item)

        select_columns: Union[Unset, List[str]] = UNSET
        if not isinstance(self.select_columns, Unset):
            select_columns = self.select_columns

        order_by_column = self.order_by_column
        order_by_descending = self.order_by_descending
        filters: Union[Unset, List[str]] = UNSET
        if not isinstance(self.filters, Unset):
            filters = self.filters

        next_ = self.next_
        agent_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.agent_ids, Unset):
            agent_ids = self.agent_ids

        device_ids: Union[Unset, List[str]] = UNSET
        if not isinstance(self.device_ids, Unset):
            device_ids = self.device_ids

        names: Union[Unset, List[str]] = UNSET
        if not isinstance(self.names, Unset):
            names = self.names

        types: Union[Unset, List[str]] = UNSET
        if not isinstance(self.types, Unset):
            types = []
            for types_item_data in self.types:
                types_item = types_item_data.value

                types.append(types_item)

        tags = self.tags
        not_names: Union[Unset, List[str]] = UNSET
        if not isinstance(self.not_names, Unset):
            not_names = self.not_names

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "parameters": parameters,
            }
        )
        if sql_query is not UNSET:
            field_dict["sqlQuery"] = sql_query
        if start is not UNSET:
            field_dict["start"] = start
        if end is not UNSET:
            field_dict["end"] = end
        if limit is not UNSET:
            field_dict["limit"] = limit
        if stream_columns is not UNSET:
            field_dict["streamColumns"] = stream_columns
        if select_columns is not UNSET:
            field_dict["selectColumns"] = select_columns
        if order_by_column is not UNSET:
            field_dict["orderByColumn"] = order_by_column
        if order_by_descending is not UNSET:
            field_dict["orderByDescending"] = order_by_descending
        if filters is not UNSET:
            field_dict["filters"] = filters
        if next_ is not UNSET:
            field_dict["next"] = next_
        if agent_ids is not UNSET:
            field_dict["agentIds"] = agent_ids
        if device_ids is not UNSET:
            field_dict["deviceIds"] = device_ids
        if names is not UNSET:
            field_dict["names"] = names
        if types is not UNSET:
            field_dict["types"] = types
        if tags is not UNSET:
            field_dict["tags"] = tags
        if not_names is not UNSET:
            field_dict["notNames"] = not_names

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.stream_column import StreamColumn

        d = src_dict.copy()
        parameters = cast(List[str], d.pop("parameters"))

        sql_query = d.pop("sqlQuery", UNSET)

        _start = d.pop("start", UNSET)
        start: Union[Unset, datetime.datetime]
        if isinstance(_start, Unset):
            start = UNSET
        else:
            start = isoparse(_start)

        _end = d.pop("end", UNSET)
        end: Union[Unset, datetime.datetime]
        if isinstance(_end, Unset):
            end = UNSET
        else:
            end = isoparse(_end)

        limit = d.pop("limit", UNSET)

        stream_columns = []
        _stream_columns = d.pop("streamColumns", UNSET)
        for stream_columns_item_data in _stream_columns or []:
            stream_columns_item = StreamColumn.from_dict(stream_columns_item_data)

            stream_columns.append(stream_columns_item)

        select_columns = cast(List[str], d.pop("selectColumns", UNSET))

        order_by_column = d.pop("orderByColumn", UNSET)

        order_by_descending = d.pop("orderByDescending", UNSET)

        filters = cast(List[str], d.pop("filters", UNSET))

        next_ = d.pop("next", UNSET)

        agent_ids = cast(List[str], d.pop("agentIds", UNSET))

        device_ids = cast(List[str], d.pop("deviceIds", UNSET))

        names = cast(List[str], d.pop("names", UNSET))

        types = []
        _types = d.pop("types", UNSET)
        for types_item_data in _types or []:
            types_item = SqlQueryTypesItem(types_item_data)

            types.append(types_item)

        tags = d.pop("tags", UNSET)

        not_names = cast(List[str], d.pop("notNames", UNSET))

        sql_query = cls(
            parameters=parameters,
            sql_query=sql_query,
            start=start,
            end=end,
            limit=limit,
            stream_columns=stream_columns,
            select_columns=select_columns,
            order_by_column=order_by_column,
            order_by_descending=order_by_descending,
            filters=filters,
            next_=next_,
            agent_ids=agent_ids,
            device_ids=device_ids,
            names=names,
            types=types,
            tags=tags,
            not_names=not_names,
        )

        sql_query.additional_properties = d
        return sql_query

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import unittest
from formant.sdk.cloud.v2.src.ingest_api import IngestAPI
from formant.sdk.cloud.v2.formant_ingest_api_client.models.ingestion_request import (
    IngestionRequest,
)
import os

EMAIL = os.getenv("FORMANT_EMAIL")
PASSWORD = os.getenv("FORMANT_PASSWORD")
BASE_URL = "https://api.formant.io/v1"


class TestIngestion(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestIngestion, self).__init__(*args, **kwargs)
        self.client = IngestAPI(email=EMAIL, password=PASSWORD, base_url=BASE_URL)

    def test_post(self):
        device_id = "404e2d2c-f95f-41d8-bce1-915e314a6898"
        tags = {}
        ingestion_request = IngestionRequest(device_id, tags)
        ingestion_request.add_numeric("test", 1)
        response = self.client.ingest.post(ingestion_request)
        self.assertEqual(response.status_code, 204)

    def test_post_all(self):
        device_id = "404e2d2c-f95f-41d8-bce1-915e314a6898"
        tags = {}

        ingestion_request_1 = IngestionRequest(device_id, tags)
        ingestion_request_1.add_numeric("test", 1)

        ingestion_request_2 = IngestionRequest(device_id, tags)
        ingestion_request_2.add_text("test_str", "str")

        ingestion_requests = [ingestion_request_1, ingestion_request_2]
        response = self.client.ingest.post_all(ingestion_requests)
        self.assertEqual(response.status_code, 204)


unittest.main()
